#include "DeerRender/UIEngine/UIEngineFunctions.h"
#include "DeerRender/UIEngine/UIEngine.h"
#include "DeerRender/UIEngine/DockPanelObject.h"
#include "Deer/Log.h"
#include "Deer/DataStore.h"

#include "angelscript.h"
#include "imgui.h"

namespace Deer {
    namespace UIEngine {
        int getMeshCount(std::string& dir) {
            return DataStore::getDirData(DEER_MESH_PATH, dir, ".dmesh").elements.size();
        }
        std::string getMeshPath(std::string& dir, int i) {
            const DirectoryData& dirData = DataStore::getDirData(DEER_MESH_PATH, dir, ".dmesh");
            if (i < 0 || i >= dirData.elements.size()){
                DEER_UI_ENGINE_ERROR("Invalid id {0} calling getMeshPath(..), mas size {1}", i, dirData.subDirs.size());
                if (currentDockPanelExecution) {
                    currentDockPanelExecution->invalidate();
                }
                return "";
            }
            return dirData.elements[i].string();
        }
        std::string getMeshName(std::string& dir, int i) {
            const DirectoryData& dirData = DataStore::getDirData(DEER_MESH_PATH, dir, ".dmesh");
            if (i < 0 || i >= dirData.elements.size()){
                DEER_UI_ENGINE_ERROR("Invalid id {0} calling getMeshName(..), mas size {1}", i, dirData.subDirs.size());
                if (currentDockPanelExecution) {
                    currentDockPanelExecution->invalidate();
                }
                return "";
            }
            return dirData.elements[i].stem().string();
        }

        int getMeshDirCount(std::string& dir) {
            return DataStore::getDirData(DEER_MESH_PATH, dir, ".dmesh").subDirs.size();
        }
        std::string getMeshDirPath(std::string& dir, int i) {
            const DirectoryData& dirData = DataStore::getDirData(DEER_MESH_PATH, dir, ".dmesh");
            if (i < 0 || i >= dirData.subDirs.size()){
                DEER_UI_ENGINE_ERROR("Invalid id {0} calling getMeshDirPath(..), mas size {1}", i, dirData.subDirs.size());
                if (currentDockPanelExecution) {
                    currentDockPanelExecution->invalidate();
                }
                return "";
            }
            return dirData.subDirs[i].string();
        }
        std::string getMeshDirName(std::string& dir, int i) {
            const DirectoryData& dirData = DataStore::getDirData(DEER_MESH_PATH, dir, ".dmesh");
            if (i < 0 || i >= dirData.subDirs.size()){
                DEER_UI_ENGINE_ERROR("Invalid id {0} calling getMeshDirName(..), mas size {1}", i, dirData.subDirs.size());
                if (currentDockPanelExecution) {
                    currentDockPanelExecution->invalidate();
                }
                return "";
            }
            return dirData.subDirs[i].filename().string();
        }

        void coloredText(float r, float g, float b, std::string& msg) {
            ImGui::TextColored(ImVec4(r, g, b, 1.0f), "%s", msg.c_str());
        }
        void text(std::string& msg) {
            ImGui::Text("%s", msg.c_str());
        }

        void drawIcon(std::string& , int size) {
            // TODO
        }

        // Interaction
        bool isMouseClicked(int mouse) {
            return ImGui::IsItemClicked(mouse);
        }

        bool isMouseDoubleClicked(int mouse) {
            return ImGui::IsMouseDoubleClicked(mouse);
        }

        void setupAutomaticColumns(int pixelSize) {
            float width = ImGui::GetWindowContentRegionWidth();

            if (width < pixelSize) {
                ImGui::Columns();
                return;
            }

            int cols = (int)(width / (pixelSize));
            float componentWidth = width / (float)cols;

            ImGui::Columns(cols, 0, false);
        }

        void endColumns() {
            ImGui::Columns();
        }
        
        void nextColumn() {
            ImGui::NextColumn();
        }

        void errorCallback(const asSMessageInfo *msg, void *param) {
          if (msg->type == asMSGTYPE_WARNING) {
            DEER_UI_ENGINE_WARN("{0} : ({1}, {2}) : {3}", msg->section, msg->row, msg->col, msg->message);
          } else if( msg->type == asMSGTYPE_INFORMATION )  {
            DEER_UI_ENGINE_ERROR("{0} : ({1}, {2}) : {3}", msg->section, msg->row, msg->col, msg->message);
          }
        }

        void print(std::string& msg) {
            DEER_UI_ENGINE_INFO("{0}", msg.c_str());
        }
    }
}